#!/usr/bin/python
# -*- coding: utf-8 -*-
##
## @author Edouard DUPIN
##
## @copyright 2012, Edouard DUPIN, all right reserved
##
## @license MPL v2.0 (see license file)
##

from realog import debug
from island import tools
from island import env
from island import config
from island import multiprocess
from island import manifest
from island import commands
import status
import os


##
## @brief Get the global description of the current action
## @return (string) the description string (fist line if reserved for the overview, all is for the specific display)
##
def help():
	return "Push the manifest delivery"

##
## @brief Add argument to the specific action
## @param[in,out] my_args (death.Arguments) Argument manager
## @param[in] section Name of the currect action
##
def add_specific_arguments(my_args, section):
	pass

##
## @brief Execute the action required.
##
## @return error value [0 .. 50] the <0 value is reserved system ==> else, what you want.
##         None : No error (return program out 0)
##         -10 : ACTION is not existing
##         -11 : ACTION execution system error
##         -12 : ACTION Wrong parameters
##
def execute(_arguments):
	for elem in _arguments:
		debug.error("pull Wrong argument: '" + elem.get_option_name() + "' '" + elem.get_arg() + "'")
	
	# check system is OK
	manifest.check_lutin_is_init()
	
	configuration = config.get_unique_config()
	
	file_source_manifest = os.path.join(env.get_island_path_manifest(), configuration.get_manifest_name())
	if os.path.exists(file_source_manifest) == False:
		debug.error("Missing manifest file : '" + str(file_source_manifest) + "'")
	
	
	elem = configuration.get_manifest_config()
	
	# Check the manifest is up to date ...
	base_display = tools.get_list_base_display(0, 0, elem)
	
	mani = manifest.Manifest(file_source_manifest)
	
	destination_branch = mani.deliver_master
	source_branch = mani.deliver_develop
	
	status.deliver_push(elem, "origin", destination_branch, source_branch, base_display)


