#!/usr/bin/python
# -*- coding: utf-8 -*-
##
## @author Edouard DUPIN
##
## @copyright 2012, Edouard DUPIN, all right reserved
##
## @license MPL v2.0 (see license file)
##


from realog import debug
from . import tools
from . import env


class LinkConfig():
	def __init__(self):
		self.source = ""
		self.destination = ""


